
let Spyro = (Spyro || {});
let Field = Spyro.Field
let Text = Spyro.Text

class Collection extends React.Component {
  constructor(props) {
    super(props);
    console.log("ON CONSTRUCT", props)
    this.url = props.url || document.location.pathname
    this.state = {
      collection: props.collection || [],
      columns: this.refreshColumns(props.collection) || [],
      page: props.page || 1,
      order: props.order || "id",
      orderDirection: props.orderDirection || "asc",
      search: "",
    }
    this.searchableFields = props.searchableFields || _.intersection(this.state.columns, ["name", "title"])
    this.perPage = props.perPage || (this.state.collection.length > 0 ? this.state.collection.length : 30)
    this.model = props.model || document.location.pathname.split("/").slice(-1)
    this.actions = props.actions || true
    this.exclude = props.exclude || []

    // Si on a pas donné de collection initiale, on va la fetcher
    if (this.state.collection.length === 0)
    {
      this.load((d) =>
      {
        this.setState({columns: this.refreshColumns(this.state.collection)})
      })
    }
  }

  refreshColumns(collection = false) {
    if (!collection)
    {
      return [];
    }
    else if (collection && collection.length > 0)
    {
      return _.keys(collection[0])
    }
  }

  getCurrentParams () {
    let additionalParams = {}
    if (!this.state)
      return {}
    if (this.state.order)
      additionalParams[(this.state.orderDirection == "asc" ? "sort" : "rev_sort")] = this.state.order
    if (this.state.search && this.state.search != "")
    {
      console.log(this.searchableFields)
      let searchFields = this.searchableFields.map((e) => `in:${e}`).join(" ")
      additionalParams["search"] = `${searchFields} ${this.state.search}`
    }
    return _.extend({
      page: this.state.page,
    }, additionalParams)
  }

  load (callback = false) {
    // console.log("load with params: ", this.getCurrentParams())
    $.signedAjax({
      method: 'GET',
      data: this.getCurrentParams(),
      url: this.url,
      dataType: 'json'
    })
    .success((data) =>
    {
      // console.log("SUCCESS ! ", data)
      // console.log(this)
      this.setState({collection: data})
      if (callback)
        callback(data)
    })
    .error((a, b, c) => {
      console.warn("ERROR", a, b, c)
      if (callback)
        callback(data)
    })
  }

  destroy (id) {
    if (id) {
      $.signedAjax({
        method: 'DELETE',
        url: `${this.url}/${id}`,
        dataType: 'json'
      })
      .success((data) => { this.setState({collection: _.reject(this.state.collection, {id: id})}) })
      .error((a, b, c) => { console.warn("ERROR", a, b, c) })
    }
  }

  nextPage () {
    this.setState({page: this.state.page + 1})
  }

  previousPage () {
    this.setState({page: this.state.page - 1})
  }

  setOrder (field) {
    // console.log("ordeer: ", field)
    if (this.state.order == field)
    {
      this.setState({orderDirection: (this.state.orderDirection == "asc" ? "desc" : "asc"), page: 1})
    }
    else
    {
      this.setState({order: field, page: 1})
    }
  }

  // ====================== Lifecycle hooks =====================

  componentDidUpdate (props, state)
  {
    let requestFields = ['page', 'order', 'orderDirection', 'search']

    console.log("[MAP] new props !", props, state)
    if (_.filter(requestFields, ((e) => state[e] != this.state[e])).length > 0)
    {
      this.load()
    }
  }

  componentDidMount () {
    console.log("mounted")
    loadImages(document);
  }

  // ==================== Rendering functions ====================


  renderSearch () {
    let onSearch = ((e) => this.setState({search: e.target.value}))
    return <input type="search" placeholder="search..." onChange={onSearch} />
  }

  renderHeader (columns, actions) {
    // console.log("renderHeader: ", columns)
    let _columns = _.without(columns, ...this.exclude)
    if (actions)
      _columns = _columns.concat(["actions"])
    return (_columns.map((e) => {
      let changeOrder = ((ev) => this.setOrder(e))
      let sortActive = (this.state.order == e && this.state.orderDirection)
      return <th onClick={changeOrder} data-field={_.kebabCase(e)} data-sort={sortActive}>{e}</th>
    }))
  }

  renderActions (item) {
    let url = `${this.url}/${item.id}`
    let editUrl = `${url}/edit`
    let deleteWrapper = ((e) => this.destroy(item.id))
    return (
      <span className="item-actions">
        <a href={url} title="show"><i className="material-icons">visibility</i></a>
        <a href={editUrl} title="edit"><i className="material-icons">create</i></a>
        <a onClick={deleteWrapper} title="delete"><i className="material-icons">delete</i></a>
      </span>
    )
  }

  renderRow (item) {
    // console.log("render row: ", item)
    let columns = _.omit(item, this.exclude)
    if (this.actions)
      columns["actions"] = this.renderActions(columns)
    return (_.map(columns, function(v, k) {
      return <td data-field={_.kebabCase(k)}><Element name={k} value={v} /></td>
    }))
  }

  render () {
    let className = `${this.model} object-collection`
    let content = this.state.collection.map((e) => {
      return (<tr>{this.renderRow(e)}</tr>)
    })
    return (
      <div className={className}>
        {this.renderSearch()}
        <Pagination nextPage={this.nextPage.bind(this)} previousPage={this.previousPage.bind(this)} total={this.state.collection.length} perPage={this.perPage} page={this.state.page}/>
        <table>
          <thead><tr>{this.renderHeader(this.state.columns, this.actions)}</tr></thead>
          <tbody>{content}</tbody>
        </table>
        <Pagination nextPage={this.nextPage.bind(this)} previousPage={this.previousPage.bind(this)} total={this.state.collection.length} perPage={this.perPage} page={this.state.page}/>
        <a onClick={this.load.bind(this)}>Refresh all</a>
      </div>
    );
  }
}


