

module Spyro
  module ActionViewExtension

    module CollectionForHelper
      module Output
        module Fields
          # Key => [class => object class ; symbol => db_type, string for pattern matching]
          @@fields = { [ActiveSupport::TimeWithZone, ""]  => {:method => :datetime},
                       [Object, /_prct$/]                 => {:method => :prct},
                       [Float, ""]                        => {:method => :float},
                       [:boolean, ""]                     => {:method => :boolean},
                       [:text, /_html$/]                  => {:method => :html},
                       [:text, /code$/]                   => {:method => :html},
                       [:text, ""]                        => {:method => :text},
                       [String, /_url$/]                  => {:method => :url},
                       #[String, /_email$/]                => {:method => :email},
                       [String, ""]                       => {:method => :string}, }

          @@fields[[CarrierWave::Uploader::Base, /_image$/]] = {:method => :cw_image} if defined? CarrierWave and defined? CarrierWave::Uploader and defined? CarrierWave::Uploader::Base
          @@fields[[CarrierWave::Uploader::Base, ""]] = {:method => :cw_file} if defined? CarrierWave and defined? CarrierWave::Uploader and defined? CarrierWave::Uploader::Base
          @@fields[[ClassyEnum::Base, ""]] = {:method => :classy_enum} if defined? ClassyEnum

          @@fields[[Object, ""]] = {:method => :default}

          delegate :l, :link_to, :mail_to, :cw_image_tag, :truncate, :strip_tags, :progress, to: :@h

          def format_default value, options
            value.to_s
          end

          def format_classy_enum value, options
            value.blank? ? "" : value.text
          end

          def format_datetime value, options
            l(value, :format => options[:format] || :datetime)
          end

          def format_cw_image value, options
            cw_image_tag value, options[:version]
          end

          def format_cw_file value, options
            link_to value.file.filename, value.url unless value.file.nil?
          end

          def format_email value, options
            mail_to value
          end

          def format_boolean value, options
            value ? "Yes" : "No"
          end

          def format_period values, options
            begin_at, end_at, current = *values
            current ||= Time.zone.now

            total, actual = (end_at.to_i - begin_at.to_i), (current.to_i - begin_at.to_i)
            current_percentage = [0, 100, (actual * 100) / total].sort.second
<<EOP
      <div class="col-md-12">
          <span class="date-before col-md-6 pull-left">#{l(begin_at, :format => options[:format] || :datetime)}</span>
          <span class="date-after col-md-6 pull-right">#{l(end_at, :format => options[:format] || :datetime)}</span>
          <div class="col-md-10 col-md-offset-1">
          #{progress current_percentage}
          </div>
        </div>
EOP

          end

          def format_progress values, options
            actual, total = *values
            current_percentage = [0, 100, (actual * 100) / total].sort.second rescue 0
<<EOP
      <div class="col-md-12">
          <div class="col-md-12 percentage-label">
            #{actual} / #{total}
          </div>
          <div class="col-md-10 col-md-offset-1">
            #{progress current_percentage}
          </div>
        </div>
EOP

          end

          def format_prct value, options
            value ? ("%.2f%%" % value) : "N/A"
          end

          def format_float value, options
            value ? ("%.2f" % value) : "N/A"
          end

          def format_url value, options
            link_to value, (value.start_with?('http') ? value : ('http://' + value))
          end

          def format_html value, options
            "<span class='has-popover' data-html='true' data-content='#{@h.send(:h, value.to_s)}'>#{@h.truncate(@h.strip_tags(HTMLEntities.new.decode(value.to_s)), :length => 40, :separator => ' ')}</span>".html_safe
          end

          def format_text value, options
            "<span class='has-popover' data-content='#{@h.send(:h, value.to_s)}'>#{@h.truncate(value.to_s, :length => 40, :separator => ' ')}</span>".html_safe
          end

          def format_string value, options
            value.to_s
          end
        end
      end
    end
  end
end
