require_relative "base"
require_relative "fields"

module Spyro
  module ActionViewExtension

    module CollectionForHelper
      module Output
        class Table < Output::Base
          delegate :link_to, :mini_button, :mini_button_success, :mini_button_danger, :mini_button_info, to: :@h

          include Fields

          def header
            @header ||= (@unicollection.meta[:header] || []) + (@unicollection.meta[:actions] ? ['Actions'] : [])
          end

          def render_header
            return "" unless @unicollection.meta[:header]
            content_tag :thead, @unicollection.meta[:html_header] do
              html = header.map do |header_sym|
                header = t(header_sym)
                header = @unicollection.meta[:block_header].call header_sym, header if @unicollection.meta[:block_header]
                "<th>#{header}</th>"
              end.join('')
              "<tr>#{html}</tr>".html_safe
            end
          end

          def default_destroy_link_attributes
            {:data => {:confirm => "Are you sure?", :method => "delete", :'confirm-title' => "Confirmation"}}
          end

          def render_cell_buttons row
            row[:meta][:actions] = {} if row[:meta][:actions].nil?

            html = if row[:meta][:actions].is_a? Hash
                     (row[:meta][:actions] || []).map do |name, link|
                       case name
                         when :show
                           mini_button_success name, link
                         when :edit
                           mini_button_info name, link
                         when :destroy
                           mini_button_danger name, link, default_destroy_link_attributes
                         else
                           mini_button name, link
                       end
                     end.join ''
                   else
                     row[:meta][:actions]
                   end

            "<td><span class='btn-group'>#{html}</span></td>"
          end

          def format_value value, elem
            return self.send("format_#{elem.as}".to_sym, value, {}) if elem.as


            @@fields.each do |keys, values|
              klass, pattern = *keys
              if (Symbol === klass and klass == elem.db_type) or (Class === klass and value.is_a? klass)
                return self.send("format_#{values[:method]}".to_sym, value, {}) if elem.name.to_s.match pattern
              end
            end
            value
          end

          def custom_value_formatted value, elem
            value
          end

          def render_body
            content = @unicollection.rows.map do |row|
              html = row[:data].map do |elem|
                value_formatted = elem.value
                value_formatted = t(value_formatted) if elem.translate
                value_formatted = format_value value_formatted, elem
                value_formatted = link_to value_formatted, elem.link if elem.link
                value_formatted = custom_value_formatted value_formatted, elem
                elem.html ? content_tag(:td, value_formatted, elem.html) : "<td>#{value_formatted}</td>"
              end.join('')
              html += render_cell_buttons row if @unicollection.meta[:actions]
              (row[:meta][:opts] and row[:meta][:opts][:html]) ? content_tag(:tr, html.html_safe, row[:meta][:opts][:html]) : "<tr>#{html}</tr>"
            end.join('').html_safe

            if @unicollection.meta[:header]
              content_tag :tbody, @unicollection.meta[:html_body] do
                content
              end
            else
              content
            end
          end

          def render_footer
            return "" unless @unicollection.meta[:footer] and @unicollection.meta[:header]

            total = header.count

            content_tag :tfoot, @unicollection.meta[:html_footer] do
              content = @unicollection.meta[:footer].map do |elem|
                colspan = [(header.count / @unicollection.meta[:footer].count.to_f).ceil, total].min
                total -= colspan
                "<td colspan='#{colspan}'>#{elem}</td>"
              end.join('')
              "<tr>#{content}</tr>".html_safe
            end
          end

          def render
            return "" if @unicollection.rows.empty?
            table_html_opts = {:class => ['table', self.class.to_s.demodulize.underscore]}.deep_merge(@unicollection.meta[:html] || {}) { |key, old, new| Array.wrap(old) + Array.wrap(new) }
            content_tag :table, table_html_opts do
              html = render_header.html_safe
              html << render_body
              html << render_footer
              html
            end
          end
        end
      end
    end
  end
end
